{#######################################################################################}
{#                                                                                     #}
{#   TIsapiConn V1.09.00  (c) 1997 Ivan Magrini (Last modification Sep 1999)           #}
{#                                                                                     #}
{#   TIsapiConn is a small Unit that allows to recieve and send Data trough the        #}
{#   ISAPI interface to IIS from Microsoft.                                            #}
{#                                                                                     #}
{#   This Unit is usefull for small ISAPI Applications with or without Database        #}
{#   Connectivity.                                                                     #}
{#                                                                                     #}
{#   Not all features are implemented yet, but you can add other features without      #}
{#   problems, for further information read the Doc's.                                 #}
{#                                                                                     #}
{#   For bugs and sugestions mail me at imagrini@icore.ch                              #}
{#                                                                                     #}
{#   This unit is released to PD, you are free to use it as you like                   #}
{#                                                                                     #}
{#   Please do not remove this Header!                                                 #}
{#                                                                                     #}
{#######################################################################################}

UNIT IsapiConn;

INTERFACE

USES
  Windows, SysUtils, Classes, IsapiVars;

CONST

  CodeChars : STRING[64] = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

  svAuthType          = 'AUTH_TYPE'; { Consts Definition for "GetServerVar"}
  svContentLen        = 'CONTENT_LENGTH';
  svContentType       = 'CONTENT_TYPE';
  svPathInfo          = 'PATH_INFO';
  svPathTrans         = 'PATH_TRANSLATED';
  svQueryString       = 'QUERY_STRING';
  svRemoteAddr        = 'REMOTE_ADDR';
  svRemoteHost        = 'REMOTE_HOST';
  svRemoteUser        = 'REMOTE_USER';
  svUnMappedUser      = 'UNMAPPED_REMOTE_USER';
  svReqMethod         = 'REQUEST_METHOD';
  svScriptName        = 'SCRIPT_NAME';
  svURL               = 'URL';
  svServerName        = 'SERVER_NAME';
  svServerPort        = 'SERVER_PORT';
  svServerPortSec     = 'SERVER_PORT_SECURE';
  svServerProtocol    = 'SERVER_PROTOCOL';
  svServerSoftware    = 'SERVER_SOFTWARE';

  svHTTPAccept        = 'HTTP_ACCEPT';
  svHTTPAuthorization = 'HTTP_AUTHORIZATION';
  svHTTPConnection    = 'HTTP_CONNECTION';
  svHTTPContentLen    = 'HTTP_CONTENT_LENGTH';
  svHTTPContentType   = 'HTTP_CONTENT_TYPE';
  svHTTPHost          = 'HTTP_HOST';
  svHTTPReferer       = 'HTTP_REFERER';
  svHTTPCookie        = 'HTTP_COOKIE';
  svHTTPServer        = 'HTTP_SERVER';
  svHTTPUserAgent     = 'HTTP_USER_AGENT';
  svAllHTTP           = 'ALL_HTTP';

  { for a complete list of response codes see http://www.w3.org/hypertext/WWW/Protocols/HTTP/HTRESP.html }

  rcOK            = '200 OK'; { default }
  rcNoResponse    = '204 No Response';
  rcDocumentMoved = '301 Document Moved';
  rcUnauthorized  = '401 Unauthorized';
  rcForbidden     = '403 Forbidden';
  rcNotFound      = '404 Not Found';

  GET = 0; { Const to use in a "CASE Methode OF" clause}
  POST = 1;

  stOK  = 0;
  stErr = 1;

TYPE

  TLogString = STRING[HSE_LOG_BUFFER_LEN - 1];

  TRequest = CLASS
  PRIVATE
    FECB             : PECB;
    FStatus          : WORD;
    FError           : STRING;
    FBuffer          : POINTER;
    FContentLen      : DWORD;
    FContentType     : STRING;
    FMethod          : WORD;
    FQueryFieldList  : TStringList;
    FFormFieldList   : TStringList;
    FCookieFieldList : TStringList;

    PROCEDURE BuildQueryFieldList;
    PROCEDURE BuildFormFieldList;
    PROCEDURE BuildCookieFieldList;

    FUNCTION GetServerVar(Name: STRING): STRING;
    FUNCTION GetQueryField(Name: STRING): STRING;
    FUNCTION GetFormField(Name: STRING): STRING;
    FUNCTION GetCookie(Name: STRING): STRING;

  PUBLIC
    CONSTRUCTOR Create(ECB: PECB); VIRTUAL;
    DESTRUCTOR  Destroy; OVERRIDE;

    PROPERTY Status                   : WORD   READ FStatus;
    PROPERTY Error                    : STRING READ FError;

    PROPERTY ContentLen               : DWORD  READ FContentLen;
    PROPERTY ContentType              : STRING READ FContentType;
    PROPERTY Method                   : WORD   READ FMethod;

    PROPERTY ServerVar[Name: STRING]  : STRING READ GetServerVar;
    PROPERTY QueryField[Name: STRING] : STRING READ GetQueryField;
    PROPERTY FormField[Name: STRING]  : STRING READ GetFormField;
    PROPERTY Cookie[Name: STRING]     : STRING READ GetCookie;

    PROPERTY Data                     : POINTER READ FBuffer;
  END;

  TResponse = CLASS
  PRIVATE
    FECB                : PECB;
    FStatus             : WORD;
    FError              : STRING;
    FBuffer             : POINTER;
    FBufferPos          : DWORD;
    FBufferSize         : DWORD;
    FContentType        : STRING;
    FResponseCode       : STRING;
    FFilterText         : BOOLEAN;
    FSkipHTMLTag        : BOOLEAN;
    FResponseHeaderSent : BOOLEAN;
    FCookieList         : TStringList;
    FHTTPHeaderList     : TStringList;

    PROCEDURE SetContentType(ContentType : STRING);
    PROCEDURE SetResponseCode(ResponseCode : STRING);

    PROCEDURE AddLine(s : STRING);

  PUBLIC
    CONSTRUCTOR Create(ECB: PECB; Size : DWORD); VIRTUAL;
    DESTRUCTOR Destroy; OVERRIDE;

    PROCEDURE ClearBuffer;
    PROCEDURE FlushBuffer;
    PROCEDURE SendBuffer(Buffer : POINTER; Size : DWORD);

    PROCEDURE Send(s : STRING);
    PROCEDURE HTMLBegin(Title : STRING; BodyBkg : STRING; BodyInit : STRING);
    PROCEDURE HTMLEnd;

    PROCEDURE SendURLRedirect(URL : STRING);
    PROCEDURE SendLocalURL(URL : STRING);
    PROCEDURE SendResponseHeader;

    PROCEDURE AddCookie(Name, Value, Expires, Path, Domain : STRING);
    PROCEDURE AddHTTPHeader(Header : STRING);

    PROPERTY Status                   : WORD   READ FStatus;
    PROPERTY Error                    : STRING READ FError;

    PROPERTY ContentType  : STRING READ FContentType  WRITE SetContentType;
    PROPERTY ResponseCode : STRING READ FResponseCode WRITE SetResponseCode;

    PROPERTY FilterText   : BOOLEAN READ FFilterText  WRITE FFilterText;
    PROPERTY SkipHTMLTag  : BOOLEAN READ FSkipHTMLTag WRITE FSkipHTMLTag;
  END;

  TIsapiConn = CLASS
  PRIVATE
    FECB                : PECB;
    FStatus             : WORD;
    FError              : STRING;
    FRequest            : TRequest;
    FResponse           : TResponse;

    PROCEDURE SetServerLogString(LogMsg : STRING);
    FUNCTION  GetServerLogString: STRING;

  PUBLIC
    CONSTRUCTOR Create(ECB: PECB; Size : DWORD); VIRTUAL;
    DESTRUCTOR Destroy; OVERRIDE;

    PROCEDURE SendError(Msg : STRING);

    PROPERTY Status          : WORD      READ FStatus;
    PROPERTY Error           : STRING    READ FError;

    PROPERTY Request         : TRequest  READ FRequest;
    PROPERTY Response        : TResponse READ FResponse;

    PROPERTY ServerLogString : STRING READ GetServerLogString WRITE SetServerLogString;
  END;


  FUNCTION  AsciiToHtml(s : STRING; SkipHTMLTag : BOOLEAN): STRING;
  FUNCTION  URLDecode(s : STRING): STRING;
  PROCEDURE URLEncode(VAR URLString : STRING; s : STRING; SetAmpersand : BOOLEAN);
  FUNCTION  Base64Encode(s : STRING): STRING;
  FUNCTION  Base64Decode(s : STRING): STRING;


{#####################################################################################}
IMPLEMENTATION

FUNCTION Base64Encode(s : STRING): STRING;
VAR Position : WORD;
    Byte0, Byte1, Byte2 : BYTE;
    Value : WORD;
    NullChars : WORD;
BEGIN
  Result := '';
  NullChars := (3-(Length(s) MOD 3)) MOD 3;
  IF NullChars = 1 THEN s := s+#0;
  IF NullChars = 2 THEN s := s+#0#0;
  Position := 1;
  WHILE Position < Length(s)+1 DO BEGIN
    Byte0 := Ord(s[Position]);
    Byte1 := Ord(s[Position+1]);
    Byte2 := Ord(s[Position+2]);
    Inc(Position,3);
    Value := (Byte0 SHR 2);
    Result := Result + CodeChars[Value+1];
    Value := (((Byte0 AND 3) SHL 4) OR (Byte1 SHR 4));
    Result := Result + CodeChars[Value+1];
    Value := (((Byte1 AND 15) SHL 2) OR (Byte2 SHR 6));
    Result := Result + CodeChars[Value+1];
    Value := Byte2 AND 63;
    Result := Result + CodeChars[Value+1];
  END;
  IF NullChars = 2 THEN Result[Length(Result)-1] := '=';
  IF NullChars > 0 THEN Result[Length(Result)] := '=';
END;

FUNCTION Base64Decode(s : STRING): STRING;
VAR Position : WORD;
    Value : WORD;
    Byte0, Byte1, Byte2 : BYTE;
    NullChars : WORD;
BEGIN
  Result := '';
  IF Length(s) MOD 4 > 0 THEN Exit;
  Position := 1;
  NullChars := 0;
  WHILE Position < Length(s)+1 DO BEGIN
    IF Pos(s[Position],CodeChars) = 0 THEN Exit;
    Byte0 := (Pos(s[Position],CodeChars)-1) SHL 2;
    Inc(Position);
    {-}
    IF Pos(s[Position],CodeChars) = 0 THEN Exit;
    Value := Pos(s[Position],CodeChars)-1;
    Byte0 := Byte0 OR (Value SHR 4);
    Byte1 := Value SHL 4;
    Inc(Position);
    {-}
    IF Pos(s[Position],CodeChars) = 0 THEN
      IF s[Position] = '=' THEN BEGIN
        s[Position] := 'A';
        Inc(NullChars);
      END ELSE Exit;
    Value := Pos(s[Position],CodeChars)-1;
    Byte1 := Byte1 OR (Value SHR 2);
    Byte2 := Value SHL 6;
    Inc(Position);
    {-}
    IF Pos(s[Position],CodeChars) = 0 THEN
      IF s[Position] = '=' THEN BEGIN
        s[Position] := 'A';
        Inc(NullChars);
      END ELSE Exit;
    Byte2 := Byte2 OR (Pos(s[Position],CodeChars)-1);
    Inc(Position);
    {-}
    Result := Result + Chr(Byte0) + Chr(Byte1) + Chr(Byte2);
  END;
  IF NullChars > 0 THEN SetLength(Result,Length(Result)-NullChars);
END;

FUNCTION URLDecode(s : STRING): STRING;
CONST HexDigits = '0123456789ABCDEF';
VAR h : STRING;

  FUNCTION HexToChar(s : STRING): CHAR;
  BEGIN
    Result := Chr((Pos(s[1],HexDigits)-1) SHL 4 + (Pos(s[2],HexDigits)-1));
  END;

BEGIN
  Result := '';
  IF s <> '' THEN BEGIN
    h := s;
    s := '';
    Trim(h);
    WHILE Pos('+',h) <> 0 DO h[Pos('+',h)] := ' ';
    WHILE Pos('%',h) > 0 DO BEGIN
     s := s+Copy(h,1,Pos('%',h)-1);
     Delete(h,1,Pos('%',h));
     s := s+HexToChar(Copy(h,1,2));
     Delete(h,1,2);
    END;
    s := s+h;
    Result := s;
  END;
END;

PROCEDURE URLEncode(VAR URLString : STRING; s : STRING; SetAmpersand : BOOLEAN);
VAR h : STRING;
BEGIN
  IF s <> '' THEN BEGIN
    h := s;
    s := '';
    Trim(h);
    WHILE Pos(' ',h) <> 0 DO h[Pos(' ',h)] := '+';
    WHILE Pos('%',h) > 0 DO BEGIN
      s := s+Copy(h,1,Pos('%',h)-1);
      Delete(h,1,Pos('%',h));
      s := s+'%25';
    END;
    h := s+h;
    s := '';
    WHILE Pos('&',h) > 0 DO BEGIN
      s := s+Copy(h,1,Pos('&',h)-1);
      Delete(h,1,Pos('&',h));
      s := s+'%26';
    END;
    s := s+h;
    IF URLString = '' THEN URLString := s
     ELSE BEGIN
       IF SetAmpersand THEN URLString := URLString+'&'+s
                       ELSE URLString := URLString+s;
     END;
  END;
END;

FUNCTION AsciiToHtml(s : STRING; SkipHTMLTag : BOOLEAN): STRING;
VAR i : INTEGER;
BEGIN
  i := 1;
  Result := '';
  WHILE i <= Length(s) DO BEGIN
    CASE s[i] OF
      '<': CASE SkipHTMLTag OF
             FALSE : Result := Result + '&lt;';
             TRUE  : BEGIN
                       IF Pos('>', STRING(pChar(@s[i]))) > 0 THEN BEGIN
                         WHILE s[i] <> '>' DO BEGIN
                           Result := Result + s[i];
                           Inc(i);
                         END;
                         Dec(i);
                       END ELSE Result := Result + s[i];
                     END;
           END;
      '>': CASE SkipHTMLTag OF
             FALSE : Result := Result + '&gt;';
             TRUE  : Result := Result + s[i];
           END;
      '"': Result := Result + '&quot;';
      '&': Result := Result + '&amp;';
      '': Result := Result + '&copy;';
      '': Result := Result + '&reg;';
      '': Result := Result + '&yen;';
      '': Result := Result + '&pound;';
      '': Result := Result + '&cent;';
      '': Result := Result + '&frac14;';
      '': Result := Result + '&frac12;';
      '': Result := Result + '&frac34;';
      '': Result := Result + '&times;';
      '': Result := Result + '&plusmn;';
      '': Result := Result + '&sect;';
      '': Result := Result + '&deg;';
      '': Result := Result + '&Agrave;';
      '': Result := Result + '&agrave;';
      '': Result := Result + '&Aacute;';
      '': Result := Result + '&aacute;';
      '': Result := Result + '&Acirc;';
      '': Result := Result + '&acirc;';
      '': Result := Result + '&Atilde;';
      '': Result := Result + '&atilde;';
      '': Result := Result + '&Ccedil;';
      '': Result := Result + '&ccedil;';
      '': Result := Result + '&ETH;';
      '': Result := Result + '&eth;';
      '': Result := Result + '&Egrave;';
      '': Result := Result + '&egrave;';
      '': Result := Result + '&Eacute;';
      '': Result := Result + '&eacute;';
      '': Result := Result + '&Ecirc;';
      '': Result := Result + '&ecirc;';
      '': Result := Result + '&Euml;';
      '': Result := Result + '&euml;';
      '': Result := Result + '&Igrave;';
      '': Result := Result + '&igrave;';
      '': Result := Result + '&Iacute;';
      '': Result := Result + '&iacute;';
      '': Result := Result + '&Icirc;';
      '': Result := Result + '&icirc;';
      '': Result := Result + '&Iuml;';
      '': Result := Result + '&iuml;';
      '': Result := Result + '&Ntilde;';
      '': Result := Result + '&ntilde;';
      '': Result := Result + '&Ograve;';
      '': Result := Result + '&ograve;';
      '': Result := Result + '&Oacute;';
      '': Result := Result + '&oacute;';
      '': Result := Result + '&Ocirc;';
      '': Result := Result + '&ocirc;';
      '': Result := Result + '&Otilde;';
      '': Result := Result + '&otilde;';
      '': Result := Result + '&szlig;';
      '': Result := Result + '&Ugrave;';
      '': Result := Result + '&ugrave;';
      '': Result := Result + '&Uacute;';
      '': Result := Result + '&uacute;';
      '': Result := Result + '&Ucirc;';
      '': Result := Result + '&ucirc;';
      '': Result := Result + '&Uuml;';
      '': Result := Result + '&uuml;';
      '': Result := Result + '&Yacute;';
      '': Result := Result + '&yacute;';
      '': Result := Result + '&yuml;';
      '': Result := Result + '&THORN;';
      '': Result := Result + '&thorn;';
      '': Result := Result + '&AElig;';
      '': Result := Result + '&aelig;';
      '': Result := Result + '&Oslash;';
      '': Result := Result + '&oslash;';
      '': Result := Result + '&Aring;';
      '': Result := Result + '&aring;';
      '': Result := Result + '&Auml;';
      '': Result := Result + '&auml;';
      '': Result := Result + '&Ouml;';
      '': Result := Result + '&ouml;';
    ELSE Result := Result + s[i];
    END;
    Inc(i);
  END;
END;

{-----------------------------------------------------------------------}
{ TRequest }

CONSTRUCTOR TRequest.Create(ECB : PECB);
VAR DataSize : DWORD;
    Data : POINTER;
    BufferSize : DWORD;
    DWSource : ^DWORD;
    DWTarget : ^DWORD;
BEGIN
  INHERITED Create;
  FECB := ECB;
  FStatus := stOK;
  FError := '';
  {-}
  FContentType := StrPas(FECB.lpszContentType);
  FContentLen  := FECB.cbTotalBytes;
  IF StrPas(FECB.lpszMethod) = 'GET' THEN FMethod := GET
                                     ELSE FMethod := POST;
  {-}
  IF FContentLen > FECB.cbAvailable THEN BEGIN  { If more Data than in lpbData then read it to a buffer }
    TRY
      FBuffer := AllocMem(FContentLen);
      Data := POINTER(DWORD(FBuffer)+FECB.cbAvailable);
      DataSize := FContentLen-FECB.cbAvailable;
      WHILE DataSize > 2048 DO BEGIN
        BufferSize := 2048;
        FECB.ReadClient(FECB.ConnID,Data,@BufferSize);
        Data := POINTER(DWORD(Data)+BufferSize);
        DataSize := DataSize-BufferSize;
      END;
      IF DataSize > 0 THEN BEGIN
        BufferSize := DataSize;
        FECB.ReadClient(FECB.ConnID,Data,@BufferSize);
      END;
      {-}
      DataSize := 24576;
      DWSource := FECB.lpbData;
      DWTarget := FBuffer;
      WHILE DataSize > 0 DO BEGIN
        DWTarget^ := DWSource^;
        Inc(DWORD(DWSource),2);
        Inc(DWORD(DWTarget),2);
        Dec(DataSize);
      END;
    EXCEPT
      ON e : EXCEPTION DO BEGIN
        FStatus := stErr;
        FError := e.Message;
      END;
    END;
  END ELSE FBuffer := FECB.lpbData;
END;

DESTRUCTOR TRequest.Destroy;
BEGIN
  IF FBuffer <> FECB.lpbData THEN FreeMem(FBuffer,FContentLen);
  IF Assigned(FQueryFieldList) THEN FQueryFieldList.Free;
  IF Assigned(FFormFieldList) THEN FFormFieldList.Free;
  IF Assigned(FCookieFieldList) THEN FCookieFieldList.Free;
  {-}
  INHERITED Destroy;
END;

PROCEDURE TRequest.BuildCookieFieldList;
VAR Cookie : STRING;
    Value  : STRING;
BEGIN
  Cookie := GetServerVar('HTTP_COOKIE');
  IF Cookie <> '' THEN BEGIN
    FCookieFieldList := TStringList.Create;
    WHILE Pos(';',Cookie) > 0 DO BEGIN
      WHILE Cookie[1] = ' ' DO Delete(Cookie,1,1);
      Value := Copy(Cookie,1,Pos(';',Cookie)-1);
      WHILE Value[Length(Value)] = ' ' DO Delete(Value,Length(Value),1);
      FCookieFieldList.Add(Value);
      Delete(Cookie,1,Pos(';',Cookie));
    END;
    IF Cookie <> '' THEN BEGIN
      WHILE Cookie[1] = ' ' DO Delete(Cookie,1,1);
      WHILE Cookie[Length(Cookie)] = ' ' DO Delete(Cookie,Length(Cookie),1);
      FCookieFieldList.Add(Cookie);
    END;
  END;
END;

PROCEDURE TRequest.BuildFormFieldList;
VAR PosCounter  : DWORD;
    ReadCounter : DWORD;
    Value       : STRING;
    p           : PChar;
BEGIN
  IF (FMethod = POST) AND (FContentLen > 0) THEN BEGIN
    FFormFieldList := TStringList.Create;
    PosCounter := 0;
    ReadCounter := 0;
    Value := '';
    p := FBuffer;
    WHILE ReadCounter < FContentLen DO BEGIN
      IF p[PosCounter] = '&' THEN BEGIN
        p[PosCounter] := #0;
        Value := StrPas(p);
        FFormFieldList.Add(Value);
        p := p+PosCounter+1;
        PosCounter := 0;
        INC(ReadCounter);
      END;
      INC(PosCounter);
      INC(ReadCounter);
    END;
    Dec(PosCounter);
    Value := p[PosCounter];
    p[PosCounter] := #0;
    Value := StrPas(p)+Value;
    FFormFieldList.Add(Value);
  END;
END;

PROCEDURE TRequest.BuildQueryFieldList;
VAR Value : STRING;
    QueryString : STRING;
BEGIN
  QueryString := StrPas(FECB.lpszQueryString);
  IF QueryString <> '' THEN BEGIN
    FQueryFieldList := TStringList.Create;
    Value := '';
    WHILE Pos('&',QueryString) > 0 DO BEGIN
      Value := Copy(QueryString,1,Pos('&',QueryString)-1);
      Delete(QueryString,1,Pos('&',QueryString));
      FQueryFieldList.Add(Value);
    END;
    FQueryFieldList.Add(QueryString);
  END;
END;

FUNCTION TRequest.GetCookie(Name : STRING): STRING;
BEGIN
  Result := '';
  IF NOT Assigned(FCookieFieldList) THEN BuildCookieFieldList;
  IF Assigned(FCookieFieldList) THEN Result := FCookieFieldList.Values[Name];
END;

FUNCTION TRequest.GetFormField(Name : STRING): STRING;
BEGIN
  Result := '';
  IF NOT Assigned(FFormFieldList) THEN BuildFormFieldList;
  IF Assigned(FFormFieldList) THEN Result := FFormFieldList.Values[Name];
END;

FUNCTION TRequest.GetQueryField(Name : STRING): STRING;
BEGIN
  Result := '';
  IF NOT Assigned(FQueryFieldList) THEN BuildQueryFieldList;
  IF Assigned(FQueryFieldList) THEN Result := FQueryFieldList.Values[Name];
END;

FUNCTION TRequest.GetServerVar(Name : STRING): STRING;
VAR Buffer : ARRAY[0..4096] OF CHAR;
    BufferLen : DWORD;
    PName : PChar;
    PValue : PChar;
BEGIN
  PName := StrAlloc(Length(Name)+1);
  StrPCopy(PName,Name);
  BufferLen := 4096;
  IF FECB.GetServerVariable(FECB.ConnID,PName,@Buffer,@BufferLen) THEN BEGIN
    PValue := @Buffer;
    Result := PValue;
  END ELSE Result := '';
  StrDispose(PName);
END;

{-----------------------------------------------------------------------}
{ TResponse }

CONSTRUCTOR TResponse.Create(ECB : PECB; Size : DWORD);
BEGIN
  INHERITED Create;
  FECB := ECB;
  FStatus := stOK;
  FError := '';
  {-}
  FContentType := 'text/html';
  FResponseCode := rcOK;
  FFilterText := FALSE;
  FSkipHTMLTag := FALSE;
  FResponseHeaderSent := FALSE;
  {-}
  FBuffer := NIL;
  FBufferPos := 0;
  FBufferSize := Size;
  TRY
    FBuffer := AllocMem(Size);
  EXCEPT
    ON e : EXCEPTION DO BEGIN
      FStatus := stErr;
      FError := e.Message;
    END;
  END;
END;

DESTRUCTOR TResponse.Destroy;
BEGIN
  IF FBuffer <> NIL THEN FreeMem(FBuffer,FBufferSize);
  IF Assigned(FHTTPHeaderList) THEN FHTTPHeaderList.Free;
  IF Assigned(FCookieList) THEN FCookieList.Free;
  {-}
  INHERITED Destroy;
END;

PROCEDURE TResponse.ClearBuffer;
BEGIN
  FBufferPos := 0;
END;

PROCEDURE TResponse.FlushBuffer;
BEGIN
  IF FBufferPos > 0 THEN
  BEGIN
    FECB.WriteClient(FECB.ConnID,FBuffer,@FBufferPos,0);
    FBufferPos := 0;
  END;
END;

PROCEDURE TResponse.SendBuffer(Buffer : POINTER; Size : DWORD);
VAR FSize : DWORD;
BEGIN
  FlushBuffer;
  FSize := Size;
  FECB.WriteClient(FECB.ConnID,Buffer,@FSize,0);
END;

PROCEDURE TResponse.SetContentType(ContentType : STRING);
BEGIN
  IF NOT FResponseHeaderSent THEN FContentType := ContentType
                             ELSE RAISE Exception.Create('ContentType can not be set after SendResponseHeader!');
END;

PROCEDURE TResponse.SetResponseCode(ResponseCode : STRING);
BEGIN
  IF NOT FResponseHeaderSent THEN FResponseCode := ResponseCode
                             ELSE RAISE Exception.Create('ResponseCode can not be set after SendResponseHeader!');
END;

PROCEDURE TResponse.AddLine(s : STRING);
VAR p : PChar;
    h : STRING;
    Delta : DWORD;
BEGIN
  WHILE (FBufferPos + DWORD(Length(s)) + 2) >= FBufferSize DO BEGIN
    Delta := FBufferSize-FBufferPos;
    h := Copy(s,1,Delta);
    Delete(s,1,Delta);
    p := FBuffer;
    p := p+FBufferPos;
    StrPCopy(p,h);
    FBufferPos := FBufferPos + Delta;
    {#}
    FECB.WriteClient(FECB.ConnID,FBuffer,@FBufferPos,0);
    FBufferPos := 0;
  END;
  s := s+#13+#10;
  p := FBuffer;
  p := p+FBufferPos;
  StrPCopy(p,s);
  FBufferPos := FBufferPos + DWORD(Length(s));
END;

PROCEDURE TResponse.Send(s : STRING);
BEGIN
  IF FFilterText THEN s := ASCIIToHTML(s,FSkipHTMLTag);
  AddLine(s);
END;

PROCEDURE TResponse.SendURLRedirect(URL : STRING);
VAR ResponseBuffer : ARRAY[0..255] OF CHAR;
BEGIN
  IF NOT FResponseHeaderSent THEN BEGIN
    StrPCopy(ResponseBuffer,URL);
    FECB.ServerSupportFunction(FECB.ConnID,HSE_REQ_SEND_URL_REDIRECT_RESP,@ResponseBuffer,NIL,NIL);
  END ELSE RAISE Exception.Create('SendURLRedirect can not be used after SendResponseHeader!');
END;

PROCEDURE TResponse.SendLocalURL(URL : STRING);
VAR ResponseBuffer : ARRAY[0..255] OF CHAR;
BEGIN
  IF NOT FResponseHeaderSent THEN BEGIN
    StrPCopy(ResponseBuffer,URL);
    FECB.ServerSupportFunction(FECB.ConnID,HSE_REQ_SEND_URL,@ResponseBuffer,NIL,NIL);
  END ELSE RAISE Exception.Create('SendURLRedirect can not be used after SendResponseHeader!');
END;

PROCEDURE TResponse.SendResponseHeader;
VAR ContentString : STRING;
    ContentStringLen : DWORD;
    ResponseBuffer : ARRAY[0..255] OF CHAR;
    ContentBuffer : ARRAY[0..255] OF CHAR;
BEGIN
  ContentString := 'Content-Type: '+FContentType+#13#10;
  IF Assigned(FCookieList) THEN ContentString := ContentString+FCookieList.Text+#13#10;
  IF Assigned(FHTTPHeaderList) THEN ContentString := ContentString+FHTTPHeaderList.Text+#13#10;
  ContentString := ContentString+#13#10;
  ContentStringLen := Length(ContentString);
  StrPCopy(ContentBuffer,ContentString);
  StrPCopy(ResponseBuffer,FResponseCode);
  FECB.ServerSupportFunction(FECB.ConnID,HSE_REQ_SEND_RESPONSE_HEADER,@ResponseBuffer,@ContentStringLen,@ContentBuffer);
  FResponseHeaderSent := TRUE;
END;

PROCEDURE TResponse.AddHTTPHeader(Header : STRING);
BEGIN
  IF NOT Assigned(FHTTPHeaderList) THEN FHTTPHeaderList := TStringList.Create;
  IF Assigned(FHTTPHeaderList) THEN FHTTPHeaderList.Add(Header);
END;

PROCEDURE TResponse.AddCookie(Name, Value, Expires, Path, Domain : STRING);
VAR Cookie : STRING;
BEGIN
  IF NOT Assigned(FCookieList) THEN FCookieList := TStringList.Create;
  IF Assigned(FCookieList) THEN BEGIN
    IF (Name <> '') AND (Value <> '') AND (Domain <> '') THEN BEGIN
      Cookie := 'Set-Cookie: '+Name+'='+Value+';';
      IF Expires <> '' THEN Cookie := Cookie+' expires='+Expires+';';
      Cookie := Cookie+' path='+Path+'; domain='+Domain;
      FCookieList.Add(Cookie);
    END;
  END;
END;

PROCEDURE TResponse.HTMLBegin(Title : STRING; BodyBkg : STRING; BodyInit : STRING);
BEGIN
  IF FFilterText THEN Title := ASCIIToHTML(Title,FSkipHTMLTag);
  Addline('<HTML>'+#13#10+'<HEAD><TITLE>'+Title+'</TITLE>'+#13#10+'</HEAD>');
  AddLine('<BODY '+BodyBkg+' '+BodyInit+'>'+#13#10);
END;

PROCEDURE TResponse.HTMLEnd;
BEGIN
  AddLine(#13#10+'</BODY>'+#13#10+'</HTML>');
END;


{-----------------------------------------------------------------------}
{ TIsapiConn }

CONSTRUCTOR TIsapiConn.Create(ECB : PECB; Size : DWORD);
BEGIN
  INHERITED Create;
  FECB := ECB;
  FStatus := stOK;
  FError := '';
  {-}
  FECB.lpszLogData[0] := #0;
  {-}
  FRequest := TRequest.Create(ECB);
  IF FRequest.Status = stErr THEN BEGIN
    FError := FRequest.Error;
    FStatus := FRequest.Status;
  END ELSE BEGIN
    FResponse := TResponse.Create(ECB,Size);
    IF FResponse.Status = stErr THEN BEGIN
      FError := FResponse.Error;
      FStatus := FResponse.Status;
    END;
  END;
END;

DESTRUCTOR TIsapiConn.Destroy;
BEGIN
  IF Assigned(FResponse) THEN FResponse.Free;
  IF Assigned(FRequest) THEN FRequest.Free;
  {-}
  INHERITED Destroy;
END;

FUNCTION TIsapiConn.GetServerLogString: STRING;
VAR p : PChar;
BEGIN
  p := FECB.lpszLogData;
  Result := p;
END;

PROCEDURE TIsapiConn.SetServerLogString(LogMsg : STRING);
BEGIN
  IF Length(LogMsg) > 79 THEN SetLength(LogMsg,79);
  StrPCopy(FECB.lpszLogData,LogMsg);
END;

PROCEDURE TIsapiConn.SendError(Msg : STRING);
VAR Len : DWORD;
    ResponseBuffer : PChar;
    ContentBuffer : PChar;
    MsgBuffer : PChar;
BEGIN
  Len := 27;
  StrPCopy(ContentBuffer,'Content-Type: text/html'+#13#10#13#10);
  StrPCopy(ResponseBuffer,'200 OK');
  FECB.ServerSupportFunction(FECB.ConnID,HSE_REQ_SEND_RESPONSE_HEADER,@ResponseBuffer,@Len,@ContentBuffer);
  Len := Length(Msg);
  StrPCopy(MsgBuffer,Msg);
  FECB.WriteClient(FECB.ConnID,MsgBuffer,@Len,0);
END;

END.

